from time import sleep

from scapy.all import *  # Scapy
from scapy.layers.dns import DNS, DNSRR
from scapy.layers.inet import IP, UDP
from scapy.layers.l2 import Ether

from dash import html, callback, dcc
import dash_bootstrap_components as dbc
from dash.dependencies import Input, Output, State
from dash import dash_table, no_update

import pandas as pd  # Pandas

import requests
from bs4 import BeautifulSoup


myMAC = get_if_hwaddr(conf.iface)  # moja MAC adresa
myIP = get_if_addr(conf.iface)  # moja IP adresa

# ulozenie anchors HTML stranky suboru do DataFrame
anchors_data = pd.DataFrame(columns=['id', 'a'])
anchors_data.set_index('id', inplace=True)
anchors_data_id = 0  # id zaznamu

# ulozenie h1,h2 HTML stranky suboru do DataFrame
h1h2_data = pd.DataFrame(columns=['id', 'h1h2'])
h1h2_data.set_index('id', inplace=True)
h1h2_data_id = 0  # id zaznamu

# ulozenie img HTML stranky suboru do DataFrame
img_data = pd.DataFrame(columns=['id', 'img'])
img_data.set_index('id', inplace=True)
img_data_id = 0  # id zaznamu

close_clicks = 0  # pomocna premenna


# vzhlad stranky
layout = dbc.Container([
    dbc.Row([
        html.H1('Sken 8 - Web Scraping')
    ]),
    dbc.Row([
        html.Table(
            style={'marginBottom': '20px'},
            children=[
                html.Tr(
                    children=[
                        html.Td(
                            style={'text-align': 'center', 'border-bottom': 'none',
                                   'border-right': '1px solid', 'width': '33%'},
                            children=[html.Div('Moja MAC adresa je:'),
                                      html.Div(className='ADResa', children=[myMAC])]),
                        html.Td(
                            style={'text-align': 'center', 'border-bottom': 'none',
                                   'border-right': '1px solid', 'width': '33%'},
                            children=[html.Div('Moja IP adresa je:'),
                                      html.Div(className='ADResa', children=[myIP])]),
                        html.Td(
                            style={'text-align': 'center', 'border-bottom': 'none', 'width': '33%'},
                            children=[html.Div('Testuje sa sieť:'),
                                      html.Div(className='ADResa', children=[myIP + '/24'])])
                    ]
                )
            ]
        )
    ]),
    dbc.Row([
		html.Div(children=[html.Table(style={'marginBottom': '20px'},
                                      children=[
                                          html.Tr(children=[
                                              html.Td(style={'border-bottom': 'none'},
                                                      children=[html.Div('Skenuj URL:')]),
                                              html.Td(style={'border-bottom': 'none'},
                                                      children=[dcc.Input(id='skenovany_URL',
                                                                          value='https://www.softengine.sk',
                                                                          style={'width': 300})])
                                          ]
                                          )
                                      ]),
						   html.Table(style={'marginBottom': '0px'},
                                      children=[
                                          html.Tr(children=[
                                              html.Td(style={'border-bottom': 'none'},
                                                      children=[html.Button(id='page8read',
                                                                            n_clicks=0,
                                                                            children=['ANALYZUJ STRÁNKU'],
                                                                            style={'margin-bottom': '10px',
                                                                                   'background-color': '#333333',
                                                                                   'color': '#ffffff'}
                                                                            )
                                                               ])
                                          ]
                                          )
                                     ])
                           ]),
        dcc.Input(id='page8pomocka', value='0', style={'display': 'none'}),
        html.Div(children=[html.H2('HTML anchors, headings, images')]),
        html.Div(children=[dash_table.DataTable(
								id='page8table1',
								fixed_rows={'headers': True, 'data': 0},
								columns=[{'id': 'a', 'name': 'anchors'}],
								style_table={'border': 'grey solid',
											 'height': 300,
											 'margin-bottom': '10px'},
								style_header={'color': 'lightgrey',
											  'backgroundColor': 'rgb(30,30,30)',
											  'fontWeight': 'bold'},
								style_cell={'color': 'white',
											'backgroundColor': 'rgb(50,50,50)',
											'border': '0px solid',
											'font-size': '1.2em',
											'textAlign': 'left',
											'min-width': '50px',
											'maxWidth': '150px',
											'whiteSpace' : 'normal',
											'height' : 'auto'},
								style_as_list_view=True,
								style_data_conditional=[{'if': {'row_index': 'odd'},
														 'backgroundColor': 'rgb(70,70,70)'},
														{'if': {'state': 'active'},
														 'backgroundColor': 'rgba(0,116,217,0.3)',
														 'border': '0px solid'}],
								page_action='none'),
								dcc.Interval(
									id='page8table1_interval',
									interval=5000,  # aktualizuje n_intervals kazdych 5s
									n_intervals=0
								),
						   dash_table.DataTable(
								id='page8table2',
								fixed_rows={'headers': True, 'data': 0},
								columns=[{'id': 'h1h2', 'name': 'headings'}],
								style_table={'border': 'grey solid',
											 'height': 300,
											 'margin-bottom': '10px'},
								style_header={'color': 'lightgrey',
											  'backgroundColor': 'rgb(30,30,30)',
											  'fontWeight': 'bold'},
								style_cell={'color': 'white',
											'backgroundColor': 'rgb(50,50,50)',
											'border': '0px solid',
											'font-size': '1.2em',
											'textAlign': 'left',
											'min-width': '50px',
											'maxWidth': '150px',
											'whiteSpace' : 'normal',
											'height' : 'auto'},
								style_as_list_view=True,
								style_data_conditional=[{'if': {'row_index': 'odd'},
														 'backgroundColor': 'rgb(70,70,70)'},
														{'if': {'state': 'active'},
														 'backgroundColor': 'rgba(0,116,217,0.3)',
														 'border': '0px solid'}],
								page_action='none'),
								dcc.Interval(
									id='page8table2_interval',
									interval=5000,  # aktualizuje n_intervals kazdych 5s
									n_intervals=0
								),
						   dash_table.DataTable(
								id='page8table3',
								fixed_rows={'headers': True, 'data': 0},
								columns=[{'id': 'img', 'name': 'images'}],
								style_table={'border': 'grey solid',
											 'height': 300,
											 'margin-bottom': '10px'},
								style_header={'color': 'lightgrey',
											  'backgroundColor': 'rgb(30,30,30)',
											  'fontWeight': 'bold'},
								style_cell={'color': 'white',
											'backgroundColor': 'rgb(50,50,50)',
											'border': '0px solid',
											'font-size': '1.2em',
											'textAlign': 'left',
											'min-width': '50px',
											'maxWidth': '150px',
											'whiteSpace' : 'normal',
											'height' : 'auto'},
								style_as_list_view=True,
								style_data_conditional=[{'if': {'row_index': 'odd'},
														 'backgroundColor': 'rgb(70,70,70)'},
														{'if': {'state': 'active'},
														 'backgroundColor': 'rgba(0,116,217,0.3)',
														 'border': '0px solid'}],
								page_action='none'),
								dcc.Interval(
									id='page8table3_interval',
									interval=5000,  # aktualizuje n_intervals kazdych 5s
									n_intervals=0
								)
            
        ])
    ])
])


# Callback funkcia pre tlacidlo - potrebna pre spravne zobrazovanie textu tlacidla pri vybere stranok
@callback([Input('page8read', 'n_clicks'),
		   Input('skenovany_URL', 'value')],
          prevent_initial_call=True)
def update_page8read(v,url):
	global anchors_data
	global anchors_data_id
	global h1h2_data
	global h1h2_data_id
	global img_data
	global img_data_id

	res = requests.get(url)
	htmlData = res.content
    
	parsedData = BeautifulSoup(htmlData, 'html.parser')
	
	anchors = parsedData.find_all('a')
	for a in anchors:
		anchors_data.loc[anchors_data_id]=str(a) #['href'])
		anchors_data_id+=1  # bol pridany novy zaznam

	h1h2 = parsedData.find_all('h1')
	for h in h1h2:
		h1h2_data.loc[h1h2_data_id]=str(h)
		h1h2_data_id+=1  # bol pridany novy zaznam

	h1h2 = parsedData.find_all('h2')
	for h in h1h2:
		h1h2_data.loc[h1h2_data_id]=str(h)
		h1h2_data_id+=1  # bol pridany novy zaznam

	images = parsedData.find_all('img')
	for i in images:
		img_data.loc[img_data_id]=str(i)
		img_data_id+=1  # bol pridany novy zaznam


# Callback funkcia pre tabulku 1 - aktualizacia kazdych 5s
@callback(Output('page8table1', 'data'),
          [Input('page8table1_interval', 'n_intervals')])
def update_page8table1(n):
    return anchors_data.to_dict('records')

# Callback funkcia pre tabulku 2 - aktualizacia kazdych 5s
@callback(Output('page8table2', 'data'),
          [Input('page8table2_interval', 'n_intervals')])
def update_page8table2(n):
    return h1h2_data.to_dict('records')

# Callback funkcia pre tabulku 3 - aktualizacia kazdych 5s
@callback(Output('page8table3', 'data'),
          [Input('page8table3_interval', 'n_intervals')])
def update_page8table3(n):
    return img_data.to_dict('records')
